# **Mobroute:** Development Guide

- [Internals: Architecture](#stronginternals-architecturestrong)
- [Internals: What the Mobroute does under-the-hood](#stronginternals-what-the-mobroute-does-under-the-hoodstrong)
- [Internals: Algorithm](#stronginternals-algorithmstrong)
- [Internals: Glossary](#stronginternals-glossarystrong)
- [Testing: Using the route_tester.sh Script to Test MDBIDs](#strongtesting-using-the-route_testersh-script-to-test-mdbidsstrong)
- [Testing: Routing Debugging Tips](#strongtesting-routing-debugging-tipsstrong)

## **Internals: Architecture**

**The Mobroute project is composed of 3 separate codebases / smaller projects:**
  - [Mobsql](http://git.sr.ht/~mil/mobsql): is a GTFS-to-SQL ETL tool which
    pulls GTFS sources from the MobilityDB into a SQLite database. It exposes
    both a CLI & a Go library (consumed by Mobroute).
  - [Mobroute](http://git.sr.ht/~mil/mobroute): is the core routing tool. This
    project handles interfacing Mobsql via its public Go API and then once the
    DB is loaded, opens the DB and performs routing calculations (currently via
    the Connection Scan Algorithm). It exposes both a CLI & a Go library (
    consumed by Transito).
  - [Transito](http://git.sr.ht/~mil/transito): is a simple Android & Linux GUI 
    app to use Mobroute. It allows querying from/to via Nominatim and
    performing routing calculations on-the-go via Mobroute's underlying
    routing mechanism (note this implicitly integrates Mobsql as well).

## **Internals: What the Mobroute does under-the-hood**
Mobroute functions as both a Go library (which applications such as
[Transito](http://git.sr.ht/~mil/transito)) utilize and a CLI (which
may be used for testing routing capabilities locally). Using the CLI
directly allows endusers and developers to by issuing a single command run
point-to-point routing requests end-to-end: from fetching the Mobility
Database catalog CSV, to then fetching GTFS source data (by specifying
a filter or MDBIDs), to database initialization, to running the routing
algorithm, and then finally formatting the response. An overview of
these steps is below.

Routing requests follow a 7-step prcess. The overall dataflow upon
submitting a routing request roughly looks like this:

- (1) **Dataload: Mobility Database CSV Fetch**
  - A HTTP request is made to fetch the Mobility Database CSV
  - This CSV contains thousands of potential GTFS sources to use
  - The CSV is imported into Mobsql's SQLite database
  - (*Handled by Mobsql*)
- (2) **Dataload: Upstream (Agency) GTFS Fetch & Load**
  - The GTFS archive(s) needed as specified by the user's 'filter'
    are translated to raw MDBIDS (e.g. in case of Transito's current
    UX design)
  - The raw MDBIDs for the Mobility Database CSV are referenced
    and each upstream GTFS Zip archive is downloaded to the filesystem.
  - Each GTFS zip archive is imported into Mobsql's SQLite database, with
    each table mirroring GTFS spec (but having a 'source' column
    added refering to MDBID - thus allowing 'multisource' storage of
    GTFS data & routing)
  - (*Handled by Mobsql*)
- (3) **Dataload: GTFS Computed Tables Calculations**
  - The routing algorithm essentially takes the input of several arrays
    as params which directly correlate with particular SQL-select statements
    pulling data from the SQLite database housing the GTFS data.
  - These routing API *has the ability to operate directly on the GTFS data*
    (e.g. with no-preprocessing) selecting directly from the [core data
    extraction logic views](https://git.sr.ht/~mil/mobroute/tree/master/item/db/dbschemaextra/schemaextra.go), 
    however that would be quite inefficient.
  - There are massive indexing and precomputation gains, and as such
    several views used by the routing API are stored in 'computed tables'
    which are essentially just a materialized table indexed by MDBID
  - In this step, the views are translated to tables as such; views
    named `_vfoo` are materialized into tables like `_cvfoo`; note
    `_cv` is short for "computed view".
  - (*Handled by Mobsql; computed tabled spec'd as ExtraSchema by Mobroute*)
- (4) **Routing API: Prep for Algorithm - Memload SQL selects**
  - This step handles SQL selecting & loading to memory the data needed
    to run the core routing algorithm.
  - "Loader" functions (see `db/load*`) translate SQL selects into the 
    arrays of structs for each datatype (Connections, Transfers, etc.)
  - At the completion of this step all data needed is in memory
- (5) **Routing API: CSA Algorithm Execution**
  - The [CSA algorithm](https://i11www.iti.kit.edu/extra/publications/dpsw-isftr-13.pdf) is
    run, passing the memload'd arrays from (4) as params to the main CSA
    function entrypoint as args.
  - The result / return value is an array of 'connections' which represents
    the most efficient route as limited down from the input array of
    connections. Each connection is essentially the 'quickest' way to
    reach each destination stop given the criteria input.
- (6) **Routing API: Decoration / Memload SQL Connections Verbose**
  - The array of connections from (5) is not a user-friendly format and lacks
    details such as stop names / latitude / longitude and similar.
  - Thus we go back to the DB as the raw GTFS data has all this information
    so we just pull the same connections (by a UID) with verbose 
    information metadata.
- (7) **Routing API: Formatting**
  - The result of (6) is formatted into different structures by pure functions
    depending on user input.
  - The main format is 'legs' which is just steps for accomplishing a route
    like walk here, take a trip here at x time etc. (in much more verbose
    and dataprocessable format ofcourse).
  - There is also a 'mapurl' formatter which translates the route into
    GeoJSON and uses a Map URL rendering service which speaks GeoJSON.

**Notes:**
  - Mobsql: Steps (1)-(3) are cached / See caching system in Mobsql
  - Mobsql: Steps (1)-(3) are stateful / Mobsql opens the DB as RW
  - Mobroute: Steps (4)-(7) are stateless / Mobroute opens the DB as RO
  - Mobroute: Currently (4)-(7) exist within a single API call for the
              routing command. In the future (4) will likely be
              memoized/cacheable in the Mobroute 'runtime / in-memory state' 
              as this is the most costly piece of the routing API.


## **Internals: Algorithm**

The core of the routing system is based on the Connection Scan Algorithm
methodology.  See the following papers for more details:

- [2013 Original CSA Paper](https://i11www.iti.kit.edu/extra/publications/dpsw-isftr-13.pdf)
- [2017 Followup CSA Paper](https://arxiv.org/pdf/1703.05997.pdf)


## **Internals: Glossary**

These abbreviations are used in the sourcecode, note explanation belows:

| Term | Abbreviation | Explanation |
| ---- | ------------ | ------- |
| **IDU** | ID Ultimate | In SQL queries and routing we distinguish between source stop_id's and the 'ultimate' parent stop. A stop IDU refers to a stops.txt entry with no parent in hierarchy. |
| **SIDU** | Source ID Ultimate | Essential IDU but also prefixed with the source e.g. as "{SOURCE}_{IDU}". |
| **MDBID**: | Mobility Database ID | Source ID from pulled from the [Mobility Database Catalog](https://github.com/MobilityData/mobility-database-catalogs). |
| **DRUTCTime**: | Date-relative UTC Time | Refers to UTC time (in seconds) relative to the input date for a query. This is used primarily in the connections loader to abstract away timezones.  |
| **Source** | | Eventually will refer to a system independent representation of the imported GTFS source; for now source is always equal to MDBID. |


## **Testing: Using the route_tester.sh Script to Test MDBIDs**
Mobroute works with potentially any GTFS source as specified in the
Mobility Database. While certain sources are tested by CI or known good,
it may be helpful (either because a source is untested, or you're too
lazy to specify routing request parameters) to determine if a source
"is good and works for routing". The `route_tester.sh` script serves
this functionality as essentially acting as a smoke testing script to
allow users to run a 'random' routing request with lax parameters to
determine if a particular MDBID can route properly.  To use this script,
after the mobroute binary is built (assuming you've section to build
the mobroute binary and then run: `./scripts/route_tester.sh MDBID`

For example:
`./scripts/route_tester.sh 1898`

## **Testing: Routing Debugging Tips**
Mobroute is tested both via unit tests & manually against a number of
actual GTFS sources noted in the README. That said, if you're performing
routing requests on different untested GTFS source and/or with different
params you may get an error if your params are too strict or the GTFS
source data is invalid, here's a number of things you can check:

- **GTFS Source Validity**:
  - Run: `mobroute msstatus -f '{mdbid: [YOURMDBIDHERE]}'`
  - Check that the `status` field is set to `active`. If its not that means
    the Mobility Database has this flagged as a non-active feed and likely is
    unusable for routing.
  - Paste the `url` field into the [GTFS Validator](https://gtfs-validator.mobilitydata.org/)
    and note any issues. If there are substantial issues, this is not likely a
    feed usable for routing.
- **Routing Parameters Relaxing**:
  - There are pragmatic defaults set for maximum walk / transfer seconds, but
    relaxing these values may help in debugging.
  - Set `max_walk_seconds` and `max_transfer_seconds` to some arbitrarily high
    value such as `99999999999`
- **Routing Debugging via Logs**:
  - Look at the logs for the routing request
  - Observe the line that says: `CSA Begin: xxx conns, yyy transfers, zzz walks origin...`
  - The number for that line should be non-0 for each item
  - If you see zeros; you can pull the SQL statements and run them manually
    to debug further
- **Pay attention to size of the GTFS archives**:
  - While there are no strict limits on the size of GTFS archives for import,
    in practice smaller sized archives will import and route faster ofcourse.
    Archives with stop_times.txt in the range of hundreds of megabytes have
    been tested and route successfully, YMMV with *very* large archives such
    as entire country GTFS aggregates (such as the Germany feed).
  - In logs you can check the MB size logged in GTFS file imports based on
    Mobsql; size is proporotional to import time.
  - Additonally, for compute cycles its helpful to understand the checksum logged
    in the line `Computed table X for source (Y Z) out of date - updating` will
    include the archive sizes as well in the form of `C_XXXM_SHA`, wherein XXX
    is the number of megabytes the source GTFS source file was.
- **Debugging via SQLite DB**:
  - Run `sqlite3 ~/.cache/mobroute/sqlite.db`
  - One example, check that the calendar for today actually produces dates: `
    - `select * from _vcaltoservice where service_date = 20231220 and source = 1898`
- **Clear Cache**:
  - If working with multiple sources and you paused the load process, it might
    not be a bad idea to clear the cache wholesale and retry again.
  - Run: `rm -rf ~/.cache/mobroute`


---

> [All Mobroute Docs](http://sr.ht/~mil/mobroute)
